(require [kodhy.macros [ss $ wc cbind amap rmap ecase]])
(import [pandas :as pd])
(import [kodhy.util [R-call]])
(import [setup [cache]])

(defn inconsistency [drinking-var model-code use-mm &optional [format? True]]

  (defmacro otp [tp &optional cols]
    `(ss obs
      (&
        (.isin $condition (if use-mm
          ["Control" "Mentor_Mother"]
          ["Control"]))
        (= $timepoint ~tp))
      ~@(if cols [cols] [])))
  (setv d (.dropna (wc (otp "5_years") (cbind
    $s $condition $nhood $preg_since_birth
    :t2_drinking (otp "5_years" drinking-var)))))
  (setv d (.merge d (wc (otp "Baseline") (cbind
    $s
    :t1_drinking (otp "Baseline" drinking-var)))))
  (when (= drinking_var "alcohol_any_preg_before_noticed")
    (setv (getl d : (qw t1_drinking t2_drinking)) (.astype (=
      (getl d : (qw t1_drinking t2_drinking)) "Drinking") int)))
  (setv ($ d t1_drinking_control) (.where ($ d t1_drinking)
    (= ($ d condition) "Control")
    0))
  (setv ($ d t1_drinking_mentored) (.where ($ d t1_drinking)
    (= ($ d condition) "Mentor_Mother")
    0))
  (setv ($ d preg_since_birth) (.astype ($ d preg_since_birth) int))
  (setv ($ d nhood) (.cat.remove-unused-categories ($ d nhood)))
  (setv d (getl d : (+
    (if use-mm (qw t1_drinking_control t1_drinking_mentored) (qw t1_drinking))
    (qw nhood preg_since_birth t2_drinking))))

  (setv result (->> d (R-call (+
     "function(d) {\n"
     model-code "\n"
     "data.frame(names(coef(m)), coef(m), confint(m, level = .95))}"))))

  (setv out (pd.Series (rmap [[_ r] (.iterrows result)]
    (if format?
      (.format "{0[1]:+.2f} [{0[2]:+.2f}, {0[3]:+.2f}]" r)
      (geti r 1)))))
  (setv out.index (amap (.decode it "ASCII") (geti result : 0)))
  (setv out (pd.concat [
    (pds-from-pairs (+
      [["(Sample size)" (len d)]]
      (if use-mm
        [
          ["T1 drinking, control" (getl out "t1_drinking_control")]
          ["T1 drinking, mentored" (getl out "t1_drinking_mentored")]]
        [["T1 drinking" (getl out "t1_drinking")]])
      [["Pregnant since birth" (getl out "preg_since_birth")]]))
    (.sort-index (.rename (get out (.str.contains out.index "nhood")) :index (λ
      (.format "{} {}"
        (cond
          [(not use-mm) "Neighborhood"]
          [(= (getl nhood-conditions (cut it (len "nhood"))) "Mentor_Mother")
            "Mentored neighborhood"]
          [T "Control neighborhood"])
        (cut it (len "nhoodn"))))))]))

  {"coef" out  "nhood_n" (.value-counts ($ d nhood))})

(setv pregdrink-retest-outcomes [
  ["Weight (kg)" "child_birthweight_kg" "After_birth" Inf]
  ["Height (cm)" "child_birthheight_cm" "After_birth" 61]
  ["Head circumference ($z$-score)" "child_head_circumference_for_age_z" "After_birth" Inf]
  ["Intelligence (MPI)" "child_kaufman_iq" "5_years" Inf]])

(defn pregdrink-retest-outcome-assocs [use-mm &optional ystd]

  (defn ngetl [obj i1 i2]
    (try
      (getl obj i1 i2)
      (except [KeyError]
        np.nan)))

  (rmap [[[_ dv dv-tp dv-cap] drinking-tp scale]
      (product pregdrink-retest-outcomes ["Baseline" "5_years"] ["discrete" "continuous"])]

    (setv cache-key (+ "pregdrink-retest-outcome-assocs "
      (.join " " (map str [(bool use-mm) (bool ystd) dv drinking-tp scale]))))
    (setv [n r-squared result] (cache cache-key (fn []
      (print dv drinking-tp scale)
      (setv o2 (if use-mm obs (ss obs (= $condition "Control"))))
      (setv d (.dropna (.apply (.groupby o2 "s") (λ
        (setv it (.set-index it "timepoint"))
        (pds-from-pairs (partition [
          "nhood" (first ($ it nhood))
          "condition" (first ($ it condition))
          "child_female" (first (.dropna ($ it child_female)))
          "preg_since_birth" (ngetl it "5_years" "preg_since_birth")
          "drinking" (ngetl it drinking-tp (if (= scale "discrete")
            "alcohol_any_preg_before_noticed"
            "alcohol_aa/day_preg_before_noticed"))
          "y" (np.minimum dv-cap (ngetl it dv-tp dv))]))))))
      (for [col (qw child_female preg_since_birth)]
        (setv (getl d : col) (.astype (getl d : col) int)))
      (when (= dv "child_head_circumference_for_age_z")
        (setv d (.drop d :columns "child_female")))
      (if (= scale "discrete")
        (setv ($ d drinking) (.astype (= ($ d drinking) "Drinking") int))
        (/= ($ d drinking) (* 2 (.std :ddof 0 ($ d drinking)))))
      (setv ($ d nhood) (.cat.remove-unused-categories
        (.astype ($ d nhood) (pd.api.types.CategoricalDtype
          :categories (. ($ obs nhood) cat categories)))))
      (setv ($ d drinking_control) (.where ($ d drinking)
        (= ($ d condition) "Control")
        0))
      (setv ($ d drinking_mentored) (.where ($ d drinking)
        (= ($ d condition) "Mentor_Mother")
        0))
      (setv d (.drop d :columns (+ ["condition"] (if use-mm
        ["drinking"]
        ["drinking_control" "drinking_mentored"]))))
      (when ystd
        (/= ($ d y) (* 2 (.std :ddof 0 ($ d y)))))
      (print "n =" (len d))
      (tuple (->> d (R-call "function(d)
         {m = lm(y ~ . + 0, data = d)
          list(nrow(d),
              summary(m)$r.squared,
              data.frame(names(coef(m)), coef(m), confint(m, level = .95)))}"))))))

    (, (, dv drinking-tp scale) (pd.DataFrame
      (np.vstack (,
        [n NaN NaN]
        [r-squared NaN NaN]
        (geti result : [1 2 3])))
      :index (+ ["n" "rsquared"] (amap (.decode it "ASCII") (geti result : 0)))
      :columns (qw point lo hi)))))

(defn pregdrink-retest-outcome-assocs-csv [use-mm &optional ystd]
  (print "outcome,time,scale,coefficient,point_estimate,confidence_limit_low,confidence_limit_high")
  (for [[[dv drinking-tp scale] df] (pregdrink-retest-outcome-assocs use-mm ystd)]
    (for [[rowname row] (.iterrows df)]
      (print :sep "," #* (amap
        (cond
          [(pd.isnull it) ""]
          [(and (numeric? it) (= (int it) it)) (int it)]
          [True it])
        [dv drinking-tp scale rowname #* row])))))
